--FILE CREATES TABLE STRUCTURE OF DATABASE

-- Dropping Tables in the correct order
----------------------
DROP TABLE IF EXISTS parameter CASCADE;
DROP TABLE IF EXISTS parameter_distribution CASCADE;
DROP TABLE IF EXISTS parameter_fraction CASCADE;
DROP TABLE IF EXISTS microenvironment CASCADE;
DROP TABLE IF EXISTS diary CASCADE;
DROP TABLE IF EXISTS diary_time CASCADE;
DROP TABLE IF EXISTS scenario CASCADE;
DROP TABLE IF EXISTS country CASCADE;
DROP TABLE IF EXISTS pollutant CASCADE;
DROP TABLE IF EXISTS subgroup CASCADE;
DROP TABLE IF EXISTS emep50_grid CASCADE;
DROP TABLE IF EXISTS emep50_grid_country CASCADE;
DROP TABLE IF EXISTS concentration CASCADE;
DROP TABLE IF EXISTS population CASCADE;
DROP TABLE IF EXISTS urban_increment CASCADE;


-----------------------------------------------------------------------------------------------------------------------
-- Creating the tables
-----------------------------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------------------------------
-- tables for storing parameter values and general descriptions
-----------------------------------------------------------------------------------------------------------------------
-- Table: parameter_distribution
-- represents distribution values for parameters
CREATE TABLE parameter_distribution
(
  parameter_distribution_id SERIAL,
  parameter_id varchar(100) NOT NULL,
  pollutant_id varchar(100),
  country_id varchar(30) NOT NULL,
  microenvironment_id varchar(100) NOT NULL,
  scenario_id varchar(100) NOT NULL,
  reference varchar(100),
  distribution_parameter varchar(100),
  value numeric,
  PRIMARY KEY(parameter_distribution_id)
);

-- Table: parameter_fraction
-- represents fractions for source parameters
CREATE TABLE parameter_fraction
(
  parameter_fraction_id SERIAL,
  parameter_id varchar(100) NOT NULL,
  country_id varchar(30) NOT NULL,
  group_id varchar(100) NOT NULL,
  scenario_id varchar(100) NOT NULL,
  reference varchar(100),
  value numeric,
  PRIMARY KEY(parameter_fraction_id)
);

-- Table: parameter
-- represents parameters, their descriptions and units
CREATE TABLE parameter
(
  parameter_id varchar(100) NOT NULL,
  equation_symbol varchar(30) NOT NULL,
  unit_of_measurement varchar(30) NOT NULL,
  microenvironment_id varchar(100) NOT NULL,
  distribution_type varchar(50),
  parameter_description varchar(200),
  PRIMARY KEY (parameter_id)
);

-- Table: pollutant
-- represents pollutants, their descriptions and units
CREATE TABLE pollutant
(
  pollutant_id varchar(100) NOT NULL,
  unit_of_measurement varchar(100) NOT NULL,
  pollutant_description varchar(200),
  PRIMARY KEY (pollutant_id)
);

-- Table: country
-- represents EU countries
CREATE TABLE country
(
  country_id varchar(30) NOT NULL,
  country_name varchar(100),
  expolis_region varchar(200),
  PRIMARY KEY (country_id)
);

-- Table: scenario
-- represents scenarios
CREATE TABLE scenario
(
  scenario_id varchar(100) NOT NULL,
  scenario_type varchar(100),
  year smallint,
  scenario_description varchar(200),
  PRIMARY KEY (scenario_id)
);

-----------------------------------------------------------------------------------------------------------------------
-- tables for MTUS data
-----------------------------------------------------------------------------------------------------------------------

-- Table: microenvironment
-- represents microenvironments and their definition
CREATE TABLE microenvironment
(
  microenvironment_id varchar(100) NOT NULL,
  microenvironment_description varchar(200),
  PRIMARY KEY (microenvironment_id)
);

-- Table: diary
-- represents MTUS diaries
CREATE TABLE diary
(
  diary_id varchar(50),
  country_id varchar(30),
  group_id varchar(100),
  age smallint,
  sex varchar(30),
  emp varchar(30),
  reference varchar(200),
  PRIMARY KEY (diary_id)
);

-- Table: diary_time
-- represents time per ME in MTUS diaries
CREATE TABLE diary_time
(
  diary_time_id SERIAL,
  diary_id varchar(50) NOT NULL,
  microenvironment_id varchar(100),
  value numeric,
  PRIMARY KEY (diary_time_id)
);

-- Table: subgroup
-- represents population subgroups
CREATE TABLE subgroup
(
  group_id varchar(10) NOT NULL,
  index smallint,
  sex varchar(30),
  emp varchar(30),
  age_min numeric,
  age_max numeric,
  PRIMARY KEY (group_id)
);


-----------------------------------------------------------------------------------------------------------------------
-- tables for EMEP50 data
-----------------------------------------------------------------------------------------------------------------------

-- Table: emep50_grid
-- represents EMEP50 grid definition
CREATE TABLE emep50_grid
(
  emep50_id varchar(30) NOT NULL,
  emep50_i smallint,
  emep50_j smallint,
  center_lat numeric,
  center_lon numeric,
  PRIMARY KEY (emep50_id)
);

-- Table: emep50_grid_country
-- represents which EMEP50 cells belong to which country
CREATE TABLE emep50_grid_country
(
  emep50_grid_country_id SERIAL,
  emep50_id varchar(30),
  country_id varchar(30),
  PRIMARY KEY (emep50_grid_country_id)
);

-- Table: concentration
-- represents concentration on EMEP50 grid
CREATE TABLE concentration
(
  concentration_id SERIAL,
  pollutant_id varchar(100),
  emep50_id varchar(30),
  scenario_id varchar(100),
  value numeric,
  PRIMARY KEY (concentration_id)
);

-- Table: population
-- represents population on EMEP50 grid
CREATE TABLE population
(
  population_id SERIAL,
  emep50_id varchar(30),
  group_id varchar(100),
  country_id varchar(30),
  year smallint,
  growthrate varchar(100),
  value numeric,
  PRIMARY KEY (population_id)
);

-- Table: urban_increment
-- represents urban_increment on the EMEP50 grid
CREATE TABLE urban_increment
(
  urban_increment_id SERIAL,
  pollutant_id varchar(100),
  emep50_id varchar(30),
  scenario_id varchar(100),
  urban_pop numeric,
  value numeric,
  PRIMARY KEY (urban_increment_id)
);

-----------------------------------------------------------------------------------------------------------------------
--add indices
-----------------------------------------------------------------------------------------------------------------------
--CREATE INDEX countryDiaryTable ON diary(country_id);
--CREATE INDEX groupDiaryTable ON diary(group_id);
CREATE INDEX idxDiary ON diary_time(diary_id);
CREATE INDEX idxCountryPop ON population(country_id);

-----------------------------------------------------------------------------------------------------------------------
-- add references and foreign keys
-----------------------------------------------------------------------------------------------------------------------
--foreign keys for parameter_distribution table
ALTER TABLE parameter_distribution  ADD FOREIGN KEY (parameter_id) REFERENCES parameter ON UPDATE CASCADE;
ALTER TABLE parameter_distribution  ADD FOREIGN KEY (pollutant_id) REFERENCES pollutant ON UPDATE CASCADE;
ALTER TABLE parameter_distribution  ADD FOREIGN KEY (microenvironment_id) REFERENCES microenvironment ON UPDATE CASCADE;
ALTER TABLE parameter_distribution  ADD FOREIGN KEY (country_id) REFERENCES country ON UPDATE CASCADE;
ALTER TABLE parameter_distribution  ADD FOREIGN KEY (scenario_id) REFERENCES scenario ON UPDATE CASCADE;

--foreign keys for parameter_fractions table
ALTER TABLE parameter_fraction ADD FOREIGN KEY (parameter_id) REFERENCES parameter ON UPDATE CASCADE;
ALTER TABLE parameter_fraction ADD FOREIGN KEY (pollutant_id) REFERENCES pollutant ON UPDATE CASCADE;
ALTER TABLE parameter_fraction ADD FOREIGN KEY (country_id) REFERENCES country ON UPDATE CASCADE;
ALTER TABLE parameter_fraction ADD FOREIGN KEY (group_id) REFERENCES subgroup ON UPDATE CASCADE;
ALTER TABLE parameter_fraction ADD FOREIGN KEY (scenario_id) REFERENCES scenario ON UPDATE CASCADE;

--foreign keys for parameter table
ALTER TABLE parameter  ADD FOREIGN KEY (microenvironment_id) REFERENCES microenvironment ON UPDATE CASCADE;

--foreign keys for diary table
ALTER TABLE diary ADD FOREIGN KEY (country_id) REFERENCES country ON UPDATE CASCADE;
ALTER TABLE diary ADD FOREIGN KEY (group_id) REFERENCES subgroup ON UPDATE CASCADE;

--foreign keys for diary_time table
ALTER TABLE diary_time ADD FOREIGN KEY (diary_id) REFERENCES diary ON UPDATE CASCADE;
ALTER TABLE diary_time ADD FOREIGN KEY (microenvironment_id) REFERENCES microenvironment ON UPDATE CASCADE;

--foreign keys for emep50_grid_country table
ALTER TABLE emep50_grid_country ADD FOREIGN KEY (country_id) REFERENCES country ON UPDATE CASCADE;
ALTER TABLE emep50_grid_country ADD FOREIGN KEY (emep50_id) REFERENCES emep50_grid ON UPDATE CASCADE;

--foreign keys for concentration table
ALTER TABLE concentration ADD FOREIGN KEY (emep50_id) REFERENCES emep50_grid ON UPDATE CASCADE;
ALTER TABLE concentration ADD FOREIGN KEY (pollutant_id) REFERENCES pollutant ON UPDATE CASCADE;
ALTER TABLE concentration ADD FOREIGN KEY (scenario_id) REFERENCES scenario ON UPDATE CASCADE;

--foreign keys for population table
ALTER TABLE population ADD FOREIGN KEY (emep50_id) REFERENCES emep50_grid ON UPDATE CASCADE;
ALTER TABLE population ADD FOREIGN KEY (country_id) REFERENCES country ON UPDATE CASCADE;
ALTER TABLE population ADD FOREIGN KEY (group_id) REFERENCES subgroup ON UPDATE CASCADE;

--foreign keys for urban_increment table
ALTER TABLE urban_increment ADD FOREIGN KEY (emep50_id) REFERENCES emep50_grid ON UPDATE CASCADE;
ALTER TABLE urban_increment ADD FOREIGN KEY (pollutant_id) REFERENCES pollutant ON UPDATE CASCADE;
ALTER TABLE urban_increment ADD FOREIGN KEY (scenario_id) REFERENCES scenario ON UPDATE CASCADE;