package de.ifgi.exposure.control;
import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.logging.Level;

import de.ifgi.exposure.datamodel.*;
import de.ifgi.exposure.io.*;
import de.ifgi.exposure.model.ExposureModel;
import de.ifgi.exposure.statistics.Summariser;


/**
 * Controller class for the data import and the exposure model
 * 
 * @author Lydia Gerharz
 *
 */

public class ModelController {
	// necessary input data for the model run
	private Metadata meta;
	private MTUScountry mtus;
	private EMEPgridCountry emep;
	private IndoorParameters [] par;
	
	// output of parameter realisations
	private ArrayList<String[]> par_realisations, gridbasedIDs;
	
	// parameters for simulation
	private SimulationParameters sim;
	private VisualisationParameters vis;
	private ArrayList<String> logs = new ArrayList<String>();
	private long startTime = 0, readingTime = 0;	
	
	public ModelController(SimulationParameters sim, VisualisationParameters vis){	
		this.sim = sim;
		this.vis = vis;
	}
	
	public ModelController(){
		
	}


	 // **** MODEL EXECUTION METHODS ****
	// method for GUI to start model run
	public void runModel(boolean outputDB){
		// **** MODEL EXECUTION ****
		if(!outputDB){
			// calculate exposure
	    	float[][] exposure = calculateExposure();

	    	//write Ascii Grid if visualisation is required
			if(sim.get_gridbased()){
				writeGridBasedResultAscii(exposure);
			}	
			
	    	// if average over realisations if necessary
	    	if(!sim.get_storeRealisationResults()){
	    		getAverages(exposure);
	    	}
	    	else
		    	writeRealisationsCsv(exposure);
						
		}	
		else{
			calculateExposureToDB();
		}
		
//		long simulationTime = System.currentTimeMillis();
//		long t2 = (simulationTime-readingTime)/1000;	
//		if(t2<240)
//			console("Running "+sim.get_iterations() + " simulations took "+ t2 + " sec.");
//		else
//			console("Running "+sim.get_iterations() + " simulations took "+ t2/60 + " min.");
//		
		writeLog();
		LogFile.logger.log(Level.INFO, "Finished model run successfully.");
		System.out.println("Finished model run.");
	}
	
	private float[][] calculateExposure(){
		console("Start model execution...");
		ExposureModel model = new ExposureModel(mtus, emep, par, sim);
		float[][] exposure = null;
		
		// catch and log exception from Exposure Model run
		try{
			exposure = model.executeModelCsv();
		}
		catch(Exception e){
			System.out.println( "Exception during model execution "+ e.getMessage());
			LogFile.logger.log(Level.SEVERE, "Exception during model execution "+ e.getMessage());
		}
		catch(OutOfMemoryError ome){
			System.out.println( "Out of memory exception during model execution. ");
			LogFile.logger.log(Level.SEVERE,"Java out of memory exception during sampling from parameter fractions. \n "
								+ome.toString());	
		}
		
		// get further information from model run
		if(sim.get_gridbased())
			gridbasedIDs = model.get_gridbased_ids();
		else{
			mtus.set_smokerRealisations(model.get_smokers());
			mtus.set_biomassRealisations(model.get_biomass_burners());
		}
		par_realisations = model.get_parameterRealisations();
		model = null;
		return exposure;
	}
	
	private void calculateExposureToDB(){
		console("Start model execution...");
		// fill simulation information into database
		DataHandler dh = new DataHandler(sim);
		dh.fillSimulationTable(sim.get_simID(), sim.get_iterations(), 
				sim.get_gridbased());
		
		ExposureModel model = new ExposureModel(mtus, emep, par, sim);
		
		// catch and log exception from Exposure Model run
		try{
		model.executeModelDB(sim);
		}
		catch(Exception e){
			System.out.println( "Exception during model execution ");
			LogFile.logger.log(Level.SEVERE, "Exception during model execution ");
		}
		catch(OutOfMemoryError ome){
			System.out.println( "Out of memory exception during model execution. ");
			//LogFile.logger.log(Level.SEVERE, "Out of memory exception during model execution. ");
			LogFile.logger.log(Level.SEVERE,"Java out of memory exception during sampling from parameter fractions. \n "
								+ome.toString());	
		}		
		model = null;
	}
	
	// **** OUTPUT WRITER METHOD ****
	// writes gridbased results to AsciiGrid files
	private void writeGridBasedResultAscii(float[][] exposure){
		String visFolder = sim.get_visFolder();
		File directory = new File(visFolder);
		if(!directory.exists())
			directory.mkdir();	
		// create objects
		AsciiGridWriter agw = new AsciiGridWriter("ETRS");
		GridResults grid_results;
		if(sim.get_subgroupResults())
			grid_results= new GridResults(sim.get_country_id(),sim.get_subgroup_array(), 
				exposure, gridbasedIDs);
		else
			grid_results = new GridResults(sim.get_country_id(),exposure, gridbasedIDs);
		ArrayList<int[]> emep_ids = grid_results.get_EMEP_ids();
		
		// EU MODEL RUN
		if(!vis.get_country_wise())
			vis.read_intersectionFile(sim.get_intersectionFile());
		// if visualisation of all countries together is required just add the data
		if(!vis.get_country_wise()&&vis.get_country_count()%29!=0){
			
			// quantiles
			if(vis.get_write_quantiles()){	
				// loop for quantile levels
				float[] quantiles = vis.get_quantile_levels();
				for(int i=0; i<quantiles.length; i++){
					ArrayList<Float> q = grid_results.get_quantiles(quantiles[i],sim.get_subgroup_array().length+1);
					vis.set_quantiles_data(sim.get_country_id(),q, quantiles[i], emep_ids);
				}
			}
			// mean and sd
			else{
				// get mean and sd for total population
				ArrayList<Float> mean_all = grid_results.get_mean(sim.get_subgroup_array().length+1);
				ArrayList<Float> sd_all = grid_results.get_standard_deviation(sim.get_subgroup_array().length+1);
				
				// add data to the visualisation parameter object
				vis.set_statistics_data(sim.get_country_id(),mean_all, sd_all, emep_ids);			
			}
		}
		
		// for final country 
		else if(!vis.get_country_wise()&&vis.get_country_count()%29==0){
			// quantiles
			if(vis.get_write_quantiles()){					
				// loop for quantile levels
				float[] quantiles = vis.get_quantile_levels();
				for(int i=0; i<quantiles.length; i++){
					ArrayList<Float> q = grid_results.get_quantiles(quantiles[i],sim.get_subgroup_array().length+1);
					vis.set_quantiles_data(sim.get_country_id(),q, quantiles[i], emep_ids);
					
					// write Ascii files for all cells
					agw.writeFileETRS(sim.get_ETRSFile(), 
							sim.get_EUvisualisationFile()+"_all_q_"+quantiles[i]+".asc", 
							vis.get_quantile_data(quantiles[i]), vis.get_ids());
				}
				
				if(vis.get_startAguila()){
					AguilaManager am = new AguilaManager(sim.get_worldShapefileETRS());
					am.startAguilaQuantiles(sim.get_visBatchFile()+"_vis_all_quantiles", sim.get_EUvisualisationFile()+"_all_q");
				}	
			}
			// mean and sd
			else{
				// get mean and sd for total population
				ArrayList<Float> mean_all = grid_results.get_mean(sim.get_subgroup_array().length+1);
				ArrayList<Float> sd_all = grid_results.get_standard_deviation(sim.get_subgroup_array().length+1);
				
				// add data to the visualisation parameter object
				vis.set_statistics_data(sim.get_country_id(),mean_all, sd_all, emep_ids);
				System.out.println("Cell length for "+vis.get_country_count()+" countries: "+vis.get_mean_data().size());
				// write Ascii files for all cells
				agw.writeFileETRS(sim.get_ETRSFile(),sim.get_EUvisualisationFile()+"_all_sd.asc", vis.get_sd_data(), vis.get_ids());
				agw.writeFileETRS(sim.get_ETRSFile(),sim.get_EUvisualisationFile()+"_all_mean.asc", vis.get_mean_data(), vis.get_ids());
			
				if(vis.get_startAguila()){
					AguilaManager am = new AguilaManager(sim.get_worldShapefileETRS());
					am.startAguilaSimple(sim.get_visBatchFile()+"_vis_all_mean", sim.get_EUvisualisationFile()+"_all_mean.asc");
					am.startAguilaSimple(sim.get_visBatchFile()+"_vis_all_sd", sim.get_EUvisualisationFile()+"_all_sd.asc");
				}
				
			}			
		}
		
		// COUNTRY WISE
		else{			
			// check which statistics and subgroups/totalPop are required
			// quantiles
			if(vis.get_write_quantiles()){	
				// quantiles for total population
				// loop for quantile levels
				float[] quantiles = vis.get_quantile_levels();
				for(int i=0; i<quantiles.length; i++){
					ArrayList<Float> q = grid_results.get_quantiles(quantiles[i],sim.get_subgroup_array().length+1);
					agw.writeFileETRS(sim.get_ETRSFile(), sim.get_visualisationFile()+"_all_q_"+quantiles[i]+".asc", q, emep_ids);
				}
				
				if(vis.get_startAguila()){
					AguilaManager am = new AguilaManager(sim.get_worldShapefileETRS());
					am.startAguilaQuantiles(sim.get_visBatchFile()+"_vis_all_quantiles", sim.get_visualisationFile()+"_all_q");
				}				
				
			}
			
			// mean and sd
			else{
				// get mean and sd for total population
				ArrayList<Float> mean_all = grid_results.get_mean(sim.get_subgroup_array().length+1);
				ArrayList<Float> sd_all = grid_results.get_standard_deviation(sim.get_subgroup_array().length+1);
				
				// write Ascii files
				agw.writeFileETRS(sim.get_ETRSFile(),sim.get_visualisationFile()+"_all_sd.asc", sd_all, emep_ids);
				agw.writeFileETRS(sim.get_ETRSFile(),sim.get_visualisationFile()+"_all_mean.asc", mean_all, emep_ids);
			
				if(vis.get_startAguila()){
					AguilaManager am = new AguilaManager(sim.get_worldShapefileETRS());
					am.startAguilaSimple(sim.get_visBatchFile()+"_vis_all_mean", sim.get_visualisationFile()+"_all_mean.asc");
					am.startAguilaSimple(sim.get_visBatchFile()+"_vis_all_sd", sim.get_visualisationFile()+"_all_sd.asc");
				}					
			}
		}		
	}
	
	private void getAverages(float[][] exposure){
    	//TODO: add summarise if sim.get_storeMean == true
		
    	Summariser sumRes = new Summariser();
    	int length = 0;
    	if(sim.get_gridbased()&&sim.get_subgroupResults())
    		length = 8*emep.get_size();
    	else if(sim.get_gridbased()&&sim.get_totalPopResults())
    		length = emep.get_size();
    	else if(sim.get_subgroupResults())
    		length = mtus.get_subgroups().size();
    	else if(sim.get_individualResults())
    		length = mtus.get_size();
    	else if(sim.get_totalPopResults())
    		length = 1;
    	ArrayList<float[]> res = sumRes.summariseExposure(exposure, length);
    	
    	// write summary results
    	writeSummariesCsv(res);
	}
	
	private void writeSummariesCsv(ArrayList<float[]> exposure){
		CsvWriter cw;
		// create resultsfolder if it does not exist
		String scenFolder = sim.get_scenarioFolder();
		File scenDir = new File(scenFolder);
		if(!scenDir.exists())
			scenDir.mkdir();	
		String resFolder = sim.get_resultFolder();
		File resDir = new File(resFolder);
		if(!resDir.exists())
			resDir.mkdir();

		if(sim.get_gridbased()&&sim.get_subgroupResults()){
			cw = new CsvWriter(sim.get_resultFile()+"subgroup_grid_meanSd.csv");
			ArrayList<String> ids = new ArrayList<String>();
			for(int i=0; i<gridbasedIDs.size(); i++){
				ids.add(gridbasedIDs.get(i)[0]+","+gridbasedIDs.get(i)[1]);
			}
			if(!cw.writeSummaryResults(exposure, "emep_id, group_id", ids)){
				console("An exception occured during writing Output Data. \n Please check the log file.");
				return;
			}						
		}
		else if(sim.get_gridbased()&&sim.get_totalPopResults()){
			cw = new CsvWriter(sim.get_resultFile()+"total_pop_grid_meanSd.csv");
			if(!cw.writeSummaryResults(exposure, "emep_id", emep.get_cellIDArray())){
				console("An exception occured during writing Output Data. \n Please check the log file.");
				return;
			}						
		}
		else if(sim.get_individualResults()){
			cw = new CsvWriter(sim.get_resultFile()+"diaries_meanSd.csv");
			ArrayList<String> ids = new ArrayList<String>();
			for(int i=0; i<mtus.get_size(); i++){
				ids.add(mtus.get_group(i)+","+mtus.get_diaryID(i)+","+
					mtus.get_smokerRealisations()[i]+","+mtus.get_biomassRealisation()[i]);
			}
			if(!cw.writeSummaryResults(exposure, "group, diary_id, ETS, biomass", ids)){
				console("An exception occured during writing Output Data. \n Please check the log file.");
				return;
			}			
		}
		else if (sim.get_subgroupResults()){
			cw = new CsvWriter(sim.get_resultFile()+"subgroups_meanSd.csv");
			ArrayList<String> ids = new ArrayList<String>();
			for(int i=0; i<meta.get_subgroup_list().size(); i++){
				ids.add(meta.get_subgroup_list().get(i).get_name());
			}
			if(!cw.writeSummaryResults(exposure, "group", ids)){
				console("An exception occured during writing Output Data. \n Please check the log file.");
				return;
			}
		}
		else if (sim.get_totalPopResults()){
			cw = new CsvWriter(sim.get_resultFile()+"total_pop_meanSd.csv");
			ArrayList<String> ids = new ArrayList<String>();
			ids.add("total_population");
			if(!cw.writeSummaryResults(exposure, "group", ids)){
				console("An exception occured during writing Output Data. \n Please check the log file.");
				return;
			}
		}

		//	writer for parameters file
		CsvWriter cwpara = new CsvWriter(sim.get_resultFile()+"para_meanSd.csv");
		if(!cwpara.writeParameters(par_realisations)){
			console("An exception occured during writing Output Parameter Data. \n Please check the log file.");
		}	
	}	
	
	// writes model realisation results to csv files
	private void writeRealisationsCsv(float[][] exposure){
		CsvWriter cw;
		// create resultsfolder if it does not exist
		String scenFolder = sim.get_scenarioFolder();
		File scenDir = new File(scenFolder);
		if(!scenDir.exists())
			scenDir.mkdir();	
		String resFolder = sim.get_resultFolder();
		File resDir = new File(resFolder);
		if(!resDir.exists())
			resDir.mkdir();

		if(sim.get_gridbased()&&sim.get_subgroupResults()){
			cw = new CsvWriter(sim.get_resultFile()+"subgroup_grid.csv");
			ArrayList<String> ids = new ArrayList<String>();
			for(int i=0; i<gridbasedIDs.size(); i++){
				ids.add(gridbasedIDs.get(i)[0]+","+gridbasedIDs.get(i)[1]);
			}
			if(!cw.writeRealisationResults(exposure, "emep_id, group_id", ids)){
				console("An exception occured during writing Output Data. \n Please check the log file.");
				return;
			}						
		}
		else if(sim.get_gridbased()&&sim.get_totalPopResults()){
			cw = new CsvWriter(sim.get_resultFile()+"total_pop_grid.csv");
			if(!cw.writeRealisationResults(exposure, "emep_id", emep.get_cellIDArray())){
				console("An exception occured during writing Output Data. \n Please check the log file.");
				return;
			}						
		}
		else if(sim.get_individualResults()){
			cw = new CsvWriter(sim.get_resultFile()+"diaries.csv");
			ArrayList<String> ids = new ArrayList<String>();
			for(int i=0; i<mtus.get_size(); i++){
				ids.add(mtus.get_group(i)+","+mtus.get_diaryID(i)+","+
					mtus.get_smokerRealisations()[i]+","+mtus.get_biomassRealisation()[i]);
			}
			if(!cw.writeRealisationResults(exposure, "group, diary_id, ETS, biomass", ids)){
				console("An exception occured during writing Output Data. \n Please check the log file.");
				return;
			}			
		}
		else if (sim.get_subgroupResults()){
			cw = new CsvWriter(sim.get_resultFile()+"subgroups.csv");
			ArrayList<String> ids = new ArrayList<String>();
			for(int i=0; i<meta.get_subgroup_list().size(); i++){
				ids.add(meta.get_subgroup_list().get(i).get_name());
			}
			if(!cw.writeRealisationResults(exposure, "group", ids)){
				console("An exception occured during writing Output Data. \n Please check the log file.");
				return;
			}
		}
		else if (sim.get_totalPopResults()){
			cw = new CsvWriter(sim.get_resultFile()+"total_pop.csv");
			ArrayList<String> ids = new ArrayList<String>();
			ids.add("total_population");
			if(!cw.writeRealisationResults(exposure, "group", ids)){
				console("An exception occured during writing Output Data. \n Please check the log file.");
				return;
			}
		}	
//			
//			
//		if(sim.get_gridbased()&&sim.get_subgroupResults()){
//			cw = new CsvWriter(sim.get_resultFile()+"subgroup_grid.csv");
//			console("Writing results to "+sim.get_resultFile()+"grid.csv");
//			if(!cw.writeGridSubgroupResults(exposure, gridbasedIDs)){
//				console("An exception occured during writing Output Data. \n Please check the log file.");
//				return;
//			}						
//		}
//		else if(sim.get_gridbased()&&sim.get_totalPopResults()){
//			cw = new CsvWriter(sim.get_resultFile()+"total_pop_grid.csv");
//			console("Writing results to "+sim.get_resultFile()+"grid.csv");
//			if(!cw.writeGridTotalPopResults(exposure, emep.get_cellIDArray())){
//				console("An exception occured during writing Output Data. \n Please check the log file.");
//				return;
//			}						
//		}
//		else if(sim.get_individualResults()){
//			cw = new CsvWriter(sim.get_resultFile()+"diaries.csv");
//			console("Writing results to "+sim.get_resultFile()+"diaries.csv");
//			if(!cw.writeIndividualResults(exposure, mtus)){
//				console("An exception occured during writing Output Data. \n Please check the log file.");
//				return;
//			}			
//		}
//		else if (sim.get_subgroupResults()){
//			cw = new CsvWriter(sim.get_resultFile()+"subgroups.csv");
//			console("Writing results to "+sim.get_resultFile()+"subgroups.csv");
//			if(!cw.writeSubgroupResults(exposure, meta.get_subgroup_list())){
//				console("An exception occured during writing Output Data. \n Please check the log file.");
//				return;
//			}
//		}
//		else if (sim.get_totalPopResults()){
//			cw = new CsvWriter(sim.get_resultFile()+"total_pop.csv");
//			console("Writing results to "+sim.get_resultFile()+"total_pop.csv");
//			if(!cw.writeTotalPopResults(exposure)){
//				console("An exception occured during writing Output Data. \n Please check the log file.");
//				return;
//			}
//		}

		//	writer for parameters file
		CsvWriter cwpara = new CsvWriter(sim.get_resultFile()+"para.csv");
		if(!cwpara.writeParameters(par_realisations)){
			console("An exception occured during writing Output Parameter Data. \n Please check the log file.");
		}	
	}	
	
	// **** INPUT READER METHODS ****	
	// method for GUI to start data download
	public boolean loadInputData(boolean readDB, boolean fillDB, boolean outputDB){
		LogFile.logger.log(Level.INFO,"Start log "+sim.get_simID()+ " "+getCurrentDateTime());
		// variables to measure calculation time
		startTime = System.currentTimeMillis();
		simulationDetails(readDB, outputDB);
		
		// Load input data from specified csv/txt files	and write into database
		if(fillDB&&!readDB)	{	
			if(!readFromCsvAndFeedDB()){
				console("An exception occured during reading Input Data. \n Please check the log file.");
				return false;
			}			
		}
		else{
			// Load input data from database			
			if(readDB)	{		
				if(!readFromDB()){
					console("An exception occured during reading Input Data. \n Please check the log file.");
					return false;
				}	
			}
			// Load input data from specified csv/txt files and do not use database
			else{						
				if(!readFromCsv()){
					console("An exception occured during reading Input Data. \n Please check the log file.");
					return false;
				}		
			}
		}
		
		// check if time use, concentration and population data is not empty
		if(emep.get_populationArray().size()==0){
			console("Error: Population file does not contain the selected country.");
			return false;
		}
		else if(emep.get_concentrationArray().size()==0){
			console("Error: Concentration file does not contain the selected country.");
			return false;
		}
		else if(mtus.get_size()==0){
			console("Error: Time use file does not contain the selected country.");
			return false;
		}
		
		readingTime = System.currentTimeMillis();	
		
//		// calculation time		
//		long t1 = (readingTime-startTime)/1000;
//		
//		if(t1<240)
//			console("Reading input data took "+ t1 + " sec.");
//		else
//			console("Reading input data took "+ t1/60 + " min.");
		return true;
	}
	
	// Read from database and fill model objects
	private boolean readFromDB(){
		console("Reading data from database...");
		DataHandler dh = new DataHandler(sim, false, true);
		
		console("Starting to retrieve metadata...");
		meta = dh.getMetadataDB();
		if(meta==null){
			console("Metadata could not be found in the database.");
			return false;
		}
		// get scenario year and name
		for(int i=0; i<meta.get_scenario_list().size(); i++){
			if(meta.get_scenario_list().get(i)[0].equals(sim.get_scenario())){
				sim.set_year(Integer.parseInt(meta.get_scenario_list().get(i)[2]));
			}
		}
		sim.set_subgroupHash(meta.get_subgroup_hashtable());
		console("Retrieved metadata from database.");

		// get MTUS object
		console("Starting to retrieve time use data...");
		mtus = dh.get_MTUSFromDB();
		if(mtus==null){
			console("Exception during reading time use data from database.");
			return false;
		}
		else if(mtus.get_size()==0){
			console("Country has not been found in the database.");
			return false;
		}
		console("Retrieved time use data from database.");
			
		// fill EMEP object
		console("Starting to retrieve EMEP data...");
		emep = dh.get_EMEPFromDB(sim.get_year()+"", meta.get_subgroup_hashtable());	
		if(emep==null){
			console("Exception during reading EMEP data from database.");
			return false;
		}
		console("Retrieved EMEP data from database.");
		
		// fill parameter objects
		console("Starting to retrieve parameter data...");
		par = dh.get_ParametersFromDB();
		if(par==null){
			console("Exception during reading parameter data from database.");
			return false;
		}
		console("Retrieved parameter data from database.");
		dh.closeDBAccessConnection();
		
		// test if data objects are filled correctly
		this.testParameters();
		this.testMTUS(0);
		this.testEMEP((emep.get_cellIDArray().size()-1));
		return true;
	}
	
	//Read from Csv, fill database and fill model objects
	private boolean readFromCsvAndFeedDB(){
		console("Reading data from Csv files...");

		// create data handler
		DataHandler dh = new DataHandler(sim, true, false);
		dh.initialiseDatabase(sim.get_databaseFile());
		
		// read csv files from local file directories
		startParsing("metadata files");
		meta = dh.readMetadataFolder(sim.get_metadataFolder());	
		if(meta==null){
			console("Exception during reading data from simple folder.");
			return false;
		}
		finishParsing("metadata files");
//		if(!dh.fillSimulationTable(sim.get_simID(), sim.get_iterations(), 
//				sim.get_gridbased())){
//			console("Exception during writing simulation information to database.");
//			return false;
//		}
			
		// download  and read EMEP grid file
		startParsing("population file");
		if(!dh.readEMEPfile(false, sim.get_emepDownloadFile())){
			// if download not possible try to read local file
			if(!dh.readEMEPfile(true, sim.get_emepDownloadFile())){
				console("Exception during reading EMEP grid data.");
				return false;
			}			
		}
		finishParsing("population file");
	
		// read MTUS files
		startParsing("Time use file");
		mtus = dh.readMTUSfile(sim.get_timeuseFile(), sim.get_microenvironments(), meta);
		mtus.set_subgroups(meta.get_subgroup_hashtable());
		finishParsing("Time use file");
		if(mtus==null){
			console("Exception during reading MTUS data from local file.");
			return false;
		}
		else if(mtus.get_size()==0){
			console("Country has not been found in the csv file.");
			return false;
		}
		
		// read population file
		startParsing("population file");
		// get scenario year
		for(int i=0; i<meta.get_scenario_list().size(); i++){
			if(meta.get_scenario_list().get(i)[0].equals(sim.get_scenario())){
				sim.set_year(Integer.parseInt(meta.get_scenario_list().get(i)[2]));
			}
		}
		sim.set_subgroupHash(meta.get_subgroup_hashtable());
		EMEPgridCountry emepC = dh.readPopulationFile(sim.get_populationFile(), sim.get_year(), meta.get_subgroup_hashtable());
		finishParsing("population file");
		if(emepC==null){
			console("Exception during reading population data from local file.");
			return false;
		}
		
		// read concentration files from folder
		startParsing("concentration files");	
		emep = dh.readConcentrationFile(sim.get_concentrationFile(), 
				sim.get_urban_incrementFile(), emepC);	
		if(emep==null){
			console("Exception during reading concentration data from local file.");
			return false;
		}
		finishParsing("concentration files");	
		
		// read parameter files
		startParsing("parameter files");
		par = dh.readParameterFiles(sim.get_parameterFile(),sim.get_parameterDistributionFile(), 
				sim.get_parameterFractionFile(), meta);
		finishParsing("parameter files");
		if(par==null){
			console("Exception during reading parameter data from local file.");
			return false;
		}
		
		dh.closeDBFeederConnection();
		
		// test if data objects are filled correctly
		this.testParameters();
		//this.testMTUS(0);
		this.testEMEP((emep.get_cellIDArray().size()-1));
		return true;
	}
	
	// Read from Csv and fill model objects
	private boolean readFromCsv(){		
		console("Read data from Csv files...");
		DataHandler dh = new DataHandler(sim, false, false);
		
		// read csv files from local file directories
		startParsing("metadata files");
		meta = dh.readMetadataFolder(sim.get_metadataFolder());
		if(meta==null){
			console("Exception during reading data from metadata folder.");
			return false;
		}
		finishParsing("metadata files");
		
		// read MTUS files
		startParsing("Time use file");
		mtus = dh.readSingleMTUSfile(sim.get_timeuseFile(), meta.get_subgroup_list(), meta.get_microenvironment_hashtable());
		mtus.set_subgroups(meta.get_subgroup_hashtable());
		
		if(mtus==null){
			console("Exception during reading MTUS data from local file.");
			return false;
		}
		else if(mtus.get_size()==0){
			console("Country has not been found in the csv file.");
			return false;
		}
		finishParsing("Time use file");
		
		// read population file
		startParsing("population file");
		// get scenario year and name
		for(int i=0; i<meta.get_scenario_list().size(); i++){
			if(meta.get_scenario_list().get(i)[0].equals(sim.get_scenario())){
				sim.set_year(Integer.parseInt(meta.get_scenario_list().get(i)[2]));
			//	sim.set_scenario_description(meta.get_scenario_list().get(i)[1]);
			}
		}
		sim.set_subgroupHash(meta.get_subgroup_hashtable());
		EMEPgridCountry emepC = dh.readPopulationFile(sim.get_populationFile(),
				sim.get_year(),meta.get_subgroup_hashtable());
		if(emepC==null){
			console("Exception during reading population data from local file.");
			return false;
		}
		finishParsing("population file");
		
		// read concentration files from folder
		startParsing("concentration file");	
		emep = dh.readConcentrationFile(sim.get_concentrationFile(), 
				sim.get_urban_incrementFile(), emepC);
		if(emep==null){
			console("Exception during reading concentration data from local.");
			return false;
		}
		finishParsing("concentration file");		
		
		// read parameter files
		startParsing("parameter files");
		par = dh.readSingleParameterFile(sim.get_parameterFile(),sim.get_parameterDistributionFile(), 
				sim.get_parameterFractionFile(),meta.get_subgroup_hashtable());
		if(par==null){
			console("Exception during reading parameter data from local file.");
			return false;
		}
		finishParsing("parameter files");
	
		// test if data objects are filled correctly
		this.testParameters();
		this.testMTUS(this.timeUseSize()-5);
		this.testEMEP((emep.get_cellIDArray().size()-1));
		return true;
	}
	
	//**** UTILITIES ****
	private void simulationDetails(boolean readDB, boolean outputDB){
		console(getCurrentDateTime());
		console("Started simulation "+sim.get_simID());
		console("--------------");
		console("Simulation details \n");
		console("Country: "+sim.get_country_id());
		console("Pollutant: "+sim.get_pollutant());
		console("Scenario: "+sim.get_scenario());
		console("Iterations: "+sim.get_iterations()+"\n");
		if(readDB)
			console("Input is read from database.");
		else
			console("Input is read from csv files.");	
		console("--------------------------------- \n");
	}

    private String getCurrentDateTime() {
        DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
        Date date = new Date();
        return dateFormat.format(date);
    }
    
	// method to write parameter content as output to the console
	private void testParameters(){	
		console("Parameter data for: "+par[0].get_country_id());
		console("--------------");
		console("Number of parameters: "+par[0].get_parameterHash().size());
		
		// go through parameter objects for different MEs
		for(int i=0; i<par.length; i++){
			Hashtable<String,float[]> paras = par[i].get_parameterHash();
			String me = par[i].get_me_id();
			console("Parameters for "+me+" environment");
			Enumeration<String> e = paras.keys();
			while(e.hasMoreElements()){
				String id = e.nextElement();
				float[] parameter = paras.get(id);
				for(int j=0; j<parameter.length; j++){
					console(id+" ["+j+"]: "+parameter[j]);
				}
			}
		}	
		console("--------------------------------- \n");
	}
	
	// method to write MTUS content as output to the console
	private void testMTUS(int individual){
		individual = 30;
		// write summary paramters from MTUS
		console("Time use data for: "+mtus.get_countryID());
		console("--------------");
		console("Diary count: "+mtus.get_size());
		console("Example data for individual "+mtus.get_diaryID(individual)+":");
		console(mtus.get_group(individual)+", MEs:"+mtus.get_time(individual, "home")
				+", "+mtus.get_time(individual, "work")+", "+mtus.get_time(individual, "travel")
				+", "+mtus.get_time(individual, "outdoor")+", "+mtus.get_time(individual, "other"));
		console("--------------------------------- \n");
	}
	
	// method to write EMEP content as output to the console
	private void testEMEP(int cell){	
		console("EMEP data for: "+emep.get_countryID());
		console("--------------");
		console("Population cell count: "+emep.get_populationArray().size());
		console("Concentration cell count: "+emep.get_concentrationArray().size());
		console("Example data for cell "+emep.get_cellID(cell));
		console("Population value F15: "+emep.get_population(cell, 0));
		console("Population value M15: "+emep.get_population(cell, 1));
		console("Population value Fwork: "+emep.get_population(cell, 2));
		console("Population value Mwork: "+emep.get_population(cell, 3));
		console("Population value Fnonwork: "+emep.get_population(cell, 4));
		console("Population value Mnonwork: "+emep.get_population(cell, 5));
		console("Population value F65: "+emep.get_population(cell, 6));
		console("Population value M65: "+emep.get_population(cell, 7));
		console("Concentration: "+emep.get_concentration(cell));
		console("Urban increment: "+emep.get_urban_increment(cell));
		console("-------------------------------------- \n");
	}
	
	public void writeLog(){
		CsvWriter cw;
		if(sim.get_gridbased())
			cw = new CsvWriter(sim.get_logFile()+"grid.txt");
		else if(sim.get_individualResults())
			cw = new CsvWriter(sim.get_logFile()+"diaries.txt");
		else
			cw = new CsvWriter(sim.get_logFile()+"subgroups.txt");
		if(!cw.writeLogFile(logs)){
			console("An exception occured during writing simulation info file. \n Please check the log file.");
		}
	}
	
	private void console(String data){
		System.out.println(data); //console output
		logs.add(data); //logfile
		if(data.endsWith("\n"))
			logs.add("");
	}
	
	private void startParsing(String file){
		System.out.println("Parsing now "+file+"...");
	}
	
	private void finishParsing(String file){
		System.out.println("Parsed "+file+" sucessfully!");
		System.out.println("-------------- \n");
	}
	
	public void resetResultTables(){
		console("Clear result tables.");
		DataHandler dh = new DataHandler(sim, true, false);
		dh.initialiseDatabase(sim.get_resetResultsDBFile());
		dh.closeDBFeederConnection();
		console("Done.");
	}

	public int timeUseSize(){
		return mtus.get_size();
	}
	
	public int gridSize(){
		return emep.get_size();
	}

}
