package de.ifgi.exposure.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.logging.Level;

import de.ifgi.exposure.control.LogFile;

public class AsciiGridWriter {
	
	private int ncols, nrows;
	private double xllcorner, yllcorner, cellsize, NODATA_value;
	private ArrayList<int[]> asciiCells;
	
	public AsciiGridWriter(String crs){
		
		// header for EMEP50
		//ncols         132 //emep_i
		//nrows         111 //emep_j
		//xllcorner     1
		//yllcorner     1
		//cellsize      1
		//NODATA_value  -999
		if(crs.contains("EMEP")){
			// set header information
			ncols = 132;
			nrows = 111;
			NODATA_value = -999;
			xllcorner = 1;
			yllcorner = 1;
			cellsize = 1;
		}
		else if(crs.contains("ETRS")){
			asciiCells = new ArrayList<int[]>();
			
		}
	}
	
	private void readIdFileETRS(String asciiFilename){
		try{
			// first read ascii etrs example file
			BufferedReader input = new BufferedReader(new FileReader(new File(
					asciiFilename)));
			
			String[] header = new String[6];
			// read header (first 6 lines)
			for(int i=0; i<6; i++){
				String[] s = input.readLine().split(" ");
				header[i] = s[s.length-1].trim();
			}
			ncols = Integer.parseInt(header[0]);
			nrows = Integer.parseInt(header[1]);			
			xllcorner = Double.parseDouble(header[2]);
			yllcorner = Double.parseDouble(header[3]);
			cellsize = Double.parseDouble(header[4]);
			NODATA_value = Double.parseDouble(header[5]);
			
			// read cells
			String row;
			while ((row = input.readLine()) != null) {			
				String[] s = row.split(" ");
				int[] cols = new int[s.length];
				// loop through coloumns
				for(int i=0; i<s.length; i++){
					cols[i] = Integer.parseInt(s[i].trim());
				}
				asciiCells.add(cols);
			}
			//System.out.println(asciiCells.size());
			input.close();
		}
		catch(Exception e){
			System.out.println("Exception during reading file: "+ asciiFilename+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during reading file: "+ asciiFilename+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
		}
	}
	
	public boolean writeFileETRS(String asciiFilename, String filename, ArrayList<Float> data, 
			ArrayList<int[]> ids){
		try{
			// read template file if this did not happen before
			if(asciiCells.isEmpty())
				this.readIdFileETRS(asciiFilename);
			
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(filename)));
			
			// write header
			writeHeader(out);
			
			// convert EMEP i and j to ascii id
			ArrayList<Integer> ascii_ids = new ArrayList<Integer>();
			for(int i=0; i<ids.size(); i++){
				int[] curr = ids.get(i);
				// emep_ids: i=col, j=row
				// asciiIds: colEMEP+(rowEMEP-1)*colsEMEP
				int currID = curr[0] + (curr[1]-1) * 132;
				//System.out.println(currID);
				ascii_ids.add(currID);
			}
			// write data
			// loop through rows
			for(int row=0; row<asciiCells.size(); row++){
				int[] currRow = asciiCells.get(row);
				
				// write current row, loop through columns
				for(int col=0; col<ncols; col++){
					boolean data_available = false;
					// if the current cell is not NoData
					if(currRow[col]!=NODATA_value){
					// check if data is available for current cell
						for(int i=0; i<ascii_ids.size(); i++){
							int currID = ascii_ids.get(i);
							
							if(currID == currRow[col]){
								// write data if it's available
								out.write(data.get(i)+" "); 
								
								// remove data from lists
								i=ascii_ids.size();
								data_available = true;
							}
						}
						if(!data_available)
							out.write(NODATA_value + " ");
					}
					else
						out.write(currRow[col]+" ");
				}
				//System.out.println("row:"+ row);
				if(row<(asciiCells.size()-1))
					out.newLine();
			}
			out.close();
			
		}
		catch(Exception e){
			System.out.println("Exception during writing file: "+ filename+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during writing file: "+ filename+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
			return false;
		}
		
		return false;
	}
	
	public boolean writeFileEMEP(String filename, ArrayList<Float> data, 
			ArrayList<int[]> ids){
		// show data for only one country
		
		try{
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(filename)));
			
			// write header
			writeHeader(out);
			
			// write data, start with upper left corner
			// loop through rows
			for(int row=nrows; row>0; row--){
				// write current row, loop through columns
				for(int col=(int)xllcorner; col<=ncols; col++){
					boolean data_available = false;
					// check if data is available for current cell
					// emep_ids: i=col, j=row
					for(int i=0; i<ids.size(); i++){
						int[] curr = ids.get(i);
						if(col==curr[0]&&row==curr[1]){
							// write data if it's available
							out.write(data.get(i)+" "); 
							
							// remove data from lists
							//emep_ids.remove(i);
							//data.remove(i);
							i=ids.size();
							data_available = true;
						}
					}
					if(!data_available)
					out.write("-999 "); 
				}
				//System.out.println("row:"+ row);
				if(row>1)
					out.newLine();
			}
			out.close();
			
		}
		catch(Exception e){
			System.out.println("Exception during writing file: "+ filename+"\n"+e.getMessage());
			LogFile.logger.log(Level.SEVERE,"Exception during writing file: "+ filename+"\n"
					+e.getMessage()+ " Cause "+ e.getCause());
			return false;
		}
		return true;
	}
	
	private void writeHeader(BufferedWriter out) throws IOException{
		out.write("ncols "+ncols); out.newLine();
		out.write("nrows "+nrows); out.newLine();
		out.write("xllcorner "+xllcorner); out.newLine();
		out.write("yllcorner "+yllcorner); out.newLine();
		out.write("cellsize "+cellsize); out.newLine();
		out.write("NODATA_value "+NODATA_value); out.newLine();
	}

	// write Ascii grid files for EU concentration and concentration
	public void writePopFile(){
		// get csv reader
		CsvHandler ch = new CsvHandler();
		
		// read file
		String popfile = "C:/LAMA/input/data/population.csv";
		//String file = "C:/LAMA/input/data/concentration.csv";
		ArrayList<Float> popdata = new ArrayList<Float>();
		ArrayList<int[]> popids = new ArrayList<int[]>();
		try{
			ArrayList<String[]> data = ch.readTable(popfile, ",");
			int groupCount = 0;
			float val = 0F;
			
			// csv columns: Emep50_i_j,Subgroup,CountryID,Year,Rate,Pop_Value
			for(int i=0; i<data.size(); i++){
				String row[] = data.get(i);
				// only use data from 2010 with growthrate m
				if(row[3].equals("2010")&&row[4].equals("m")){
					val += Float.parseFloat(row[5].trim());
					groupCount ++;
				}
				
				// if subgroups are filled
				if(groupCount==8){
					// get EMEP id 			
					String[] s = row[0].split("_");
					if(s.length==2){
						int[] ids = new int[2];
						ids[0] = Integer.parseInt(s[0].trim());
						ids[1] = Integer.parseInt(s[1].trim());
						// check if EMEP id already exists in list	
						boolean newValue = true;
						for(int j=0; j<popids.size(); j++){
							// if data exists already update data
							if(popids.get(j)[0]==ids[0]&&popids.get(j)[1]==ids[1]){
								float newVal = popdata.get(j) + val;
								popdata.set(j,newVal);
								newValue = false;
							}
						}
						if(newValue){
							popdata.add(val);
							popids.add(ids);
						}
					}
					groupCount = 0;
					val = 0F;
				}
			}
			// write population grid
			this.writeFileETRS("C:/LAMA/output/vis/aaigrid_etrs.txt", "C:/LAMA/output/vis/population2010BAU.asc", popdata, popids);
		}
		catch(Exception e){			
		}
	}

	public void writeConFile(){
		// get csv reader
		CsvHandler ch = new CsvHandler();
		
		// read file
		String confile = "C:/LAMA/input/data/concentration.csv";
		ArrayList<Float> condata = new ArrayList<Float>();
		ArrayList<int[]> conids = new ArrayList<int[]>();
		try{
			ArrayList<String[]> data = ch.readTable(confile, ",");
			
			// new columns: EmepID,Emep_i,Emep_j, scenario_id, Pollutant,Pol_Value
			for(int i=0; i<data.size(); i++){
				String row[] = data.get(i);
				
				// only use data for scenario 0
				if(row[3].trim().equals("0")){
					condata.add(Float.parseFloat(row[5].trim()));
					
					int[] ids = new int[2];
					ids[0] = Integer.parseInt(row[1].trim());
					ids[1] = Integer.parseInt(row[2].trim());
					conids.add(ids);
				}
				
			}
			// write concentration grid
			this.writeFileETRS("C:/LAMA/output/vis/aaigrid_etrs.txt", "C:/LAMA/output/vis/concentration2010BAU.asc", condata, conids);
		}
		catch(Exception e){	
		}			
	}
	
	public void writeUIFile(){
		// get csv reader
		CsvHandler ch = new CsvHandler();
		
		// read file
		String uifile = "C:/LAMA/input/data/urban_increment.csv";
		ArrayList<Float> uidata = new ArrayList<Float>();
		ArrayList<int[]> uiids = new ArrayList<int[]>();
		try{
			ArrayList<String[]> data = ch.readTable(uifile, ",");
			
			//i50_j50,scenario_id,PM25_UrbIn,urban_pop,rural_pop,urban_perc,rural_perc
			for(int i=0; i<data.size(); i++){
				String row[] = data.get(i);
				
				// only use data for scenario 0
				if(row[1].trim().equals("0")){
					String[] s = row[0].split("_");
					if(s.length==2){
						int[] ids = new int[2];
						ids[0] = Integer.parseInt(s[0].trim());
						ids[1] = Integer.parseInt(s[1].trim());
						uiids.add(ids);
						uidata.add(Float.parseFloat(row[2].trim()));
					}				
				}		
			}
			// write concentration grid
			this.writeFileETRS("C:/LAMA/output/vis/aaigrid_etrs.txt", "C:/LAMA/output/vis/urban_increment2010BAU.asc", uidata, uiids);
		}
		catch(Exception e){	
		}			
	}
}
