package de.ifgi.exposure.model;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.logging.Level;
import java.lang.OutOfMemoryError;

import de.ifgi.exposure.datamodel.*;
import de.ifgi.exposure.db.DatabaseFeeder;
import de.ifgi.exposure.control.LogFile;
import de.ifgi.exposure.statistics.*;

/**
 * Model for individual exposure
 * 
 * @author Lydia Gerharz
 * 
 */

public class ExposureModel {
	private MTUScountry mtus;
	private EMEPgridCountry emep;
	private IndoorParameters home, work, travel;
	private SimulationParameters sim;

	private float[] total_pop;
	private float sum_pop;
	private float[] smokers, biomass_burners;
	private ArrayList<String[]> parameter_realisations, gridbasedIDs;

	// private static Logger log = Logger.getLogger(ExposureModel.class);

	// constructor with all necessary data
	public ExposureModel(MTUScountry mtus, EMEPgridCountry emep,
			IndoorParameters[] parameters, SimulationParameters sim) {
		this.mtus = mtus;
		this.emep = emep;
		this.sim = sim;
		for (int i = 0; i < parameters.length; i++) {
			if (parameters[i].get_me_id().startsWith("h"))
				home = parameters[i];
			else if (parameters[i].get_me_id().startsWith("w"))
				work = parameters[i];
			else
				travel = parameters[i];
		}
	}

	// simple constructor
	public ExposureModel() {
	}

	// **** MAIN MODEL METHODS ****
	public void executeModelDB(SimulationParameters sim)
			throws OutOfMemoryError {
		try {
			int iterations = sim.get_iterations();
			// structure for database
			String par_real_columns = "simulation_id, parameter_id, microenvironment_id, iteration, value";
			String ind_results_columns = "simulation_id, diary_id, group_id, iteration, value";
			String grid_results_columns = "simulation_id, emep_id, group_id, iteration, value";
			DatabaseFeeder df = new DatabaseFeeder(sim.get_JDBC_Driver(), sim
					.get_dbURL(), sim.get_dbUser(), sim.get_dbPassword());
			df.connect();

			// array with total population for all subgroups
			if (!sim.get_gridbased())
				total_pop = this.sumPop();
			// get assignment for each individual if he/she is smoker, biomass
			// burner and if
			// he/she lives in an urban area for each grid cell
			float[][] i_urb = fractionSampling();

			// **** RUN SIMULATION ****
			// loop through realisations
			for (int i = 0; i < iterations; i++) {
				Hashtable<String, Float> realisations = distributionSampling();

				// sample parameter values for this realisation
				float a_work = sampling(work.get_parameter("a"), work
						.get_parameter_distribution("a"),false);
				float a_home = sampling(home.get_parameter("a"), home
						.get_parameter_distribution("a"),false);
				float v_work = sampling(work.get_parameter("v"), work
						.get_parameter_distribution("v"),false);
				float v_home = sampling(home.get_parameter("v"), home
						.get_parameter_distribution("v"),false);
				float p_work = sampling(work.get_parameter("p"), work
						.get_parameter_distribution("p"),false);
				float p_home = sampling(home.get_parameter("p"), home
						.get_parameter_distribution("p"),false);
				float k = sampling(home.get_parameter("k"), home
						.get_parameter_distribution("k"),false);
				float e = sampling(travel.get_parameter("e"), travel
						.get_parameter_distribution("e"),false);
				float s_bgr = sampling(home.get_parameter("s_bgr"), home
						.get_parameter_distribution("s_bgr"),false);
				float s_cig = sampling(home.get_parameter("s_cig"), home
						.get_parameter_distribution("s_cig"),false);
				float s_bio = sampling(home.get_parameter("s_bio"), home
						.get_parameter_distribution("s_bio"),false);
				// dividing n_cig by 24 to get number of cig smoked per hour
				float n_cig = sampling(home.get_parameter("n_cig"), home
						.get_parameter_distribution("n_cig"),false) / 24;

				// if calculation is without indoor sources, set sources to 0
				if(sim.get_exBackground())
					s_bgr = 0;
				if(sim.get_exBiomass())
					s_bio = 0;
				if(sim.get_exETS())
					s_cig = 0;
				// INSERT INTO parameter_realisation (simulation_id,
				// parameter_id, microenvironment_id, iteration, value)
				// add realisation values to array
				parameter_realisations = new ArrayList<String[]>();
				parameter_realisations.add(buildParString("a", "home", i,
						a_home));
				parameter_realisations.add(buildParString("a", "work", i,
						a_work));
				parameter_realisations.add(buildParString("v", "home", i,
						v_home));
				parameter_realisations.add(buildParString("v", "work", i,
						v_work));
				parameter_realisations.add(buildParString("p", "home", i,
						p_home));
				parameter_realisations.add(buildParString("p", "work", i,
						p_work));
				parameter_realisations.add(buildParString("k", "all", i, k));
				parameter_realisations.add(buildParString("e", "all", i, e));
				parameter_realisations.add(buildParString("s_bgr", "home", i,
						s_bgr));
				parameter_realisations.add(buildParString("s_bio", "home", i,
						s_bio));
				parameter_realisations.add(buildParString("s_cig", "home", i,
						s_cig));
				parameter_realisations.add(buildParString("n_cig", "home", i,
						n_cig));

				// write parameter realisations to database
				try {
					df.insertRows(parameter_realisations, par_real_columns,
							"parameter_realisation");
				} catch (Exception ex) {
					LogFile.logger
							.log(Level.SEVERE,
									"Exception during inserting parameter realisations into table \n "
											+ ex.toString() + " Cause "
											+ ex.getCause());
				}
				// **** LOOP THROUGH GRID CELLS ****
				if (sim.get_gridbased()) {
					// build array with sources for each individual
					float[] s_home = new float[smokers.length];
					for (int l = 0; l < smokers.length; l++) {
						s_home[l] = s_bgr + (float) smokers[l] * s_cig * n_cig
								+ (float) biomass_burners[l] * s_bio;
						s_home[l] = realisations.get("s_bgr")
								+ (float) smokers[l]
								* realisations.get("s_cig")
								* realisations.get("n_cig")
								+ (float) biomass_burners[l]
								* realisations.get("s_bio");
					}

					ArrayList<String[]> gridResults = new ArrayList<String[]>();
					for (int j = 0; j < emep.get_cellIDArray().size(); j++) {
						// calculate exposure per grid cell

						float[] home_all = this.modelIndoorConcGrid("home",
								a_home, k, p_home, v_home, s_home[j], i_urb, j);
						float[] work_all = this.modelIndoorConcGrid("work",
								a_work, k, p_work, v_work, s_bgr, i_urb, j);

						float[] outdoor = this.modelOutdoorConcGrid("outdoor",
								1, i_urb, j);
						float[] travel = this.modelOutdoorConcGrid("travel", e,
								i_urb, j);

						// get total exposure for each subgroup
						for (int l = 0; l < 8; l++) {
							double exp = home_all[l] + work_all[l] + travel[l]
									+ outdoor[l];
							// INSERT INTO exposure_grid_realisation
							// (simulation_id, emep50_id, group_id, iteration,
							// value)
							String[] tmp = { sim.get_simID(),
									emep.get_cellID(j),
									mtus.get_subgroups().get((l + 1) + ""),
									(i + 1 + ""), (exp + "") };
							//System.out.println(tmp[0] + ", " + tmp[1] + ", "
							//		+ tmp[2] + ", " + tmp[3] + ", " + tmp[4]);
							gridResults.add(tmp);
						}
					}
					try {
						// write grid results for this iteration to database
						df.insertRows(gridResults, grid_results_columns,
								"exposure_grid_realisation");
					} catch (Exception ex) {
						LogFile.logger.log(Level.SEVERE,
								"Exception during inserting exposure grid results into table \n "
										+ ex.toString() + " Cause "
										+ ex.getCause());
					}
				}

				// **** LOOP THROUGH INDIVIDUALS ****
				else {
					ArrayList<String[]> individualResults = new ArrayList<String[]>();
					for (int j = 0; j < mtus.get_size(); j++) {
						try {
							float s_home = 0, home = 0, work = 0, outdoor = 0, travel = 0;
							// calculate home source for each individual
							// depending if biomass or smoking occurs
							s_home = s_bgr + (float) smokers[j] * s_cig * n_cig
									+ (float) biomass_burners[j] * s_bio;

							// get exposure per environment
							// assign 50 % of the other indoor time to indoors
							home = this.modelIndoorConcPop((int) (mtus
									.get_time(j, "home") + 0.5 * mtus.get_time(
									j, "other")), a_home, k, p_home, v_home,
									s_home, i_urb[j], mtus.get_groupID(j));

							work = this.modelIndoorConcPop(mtus.get_time(j,
									"work"), a_work, k, p_work, v_work, s_bgr,
									i_urb[j], mtus.get_groupID(j));

							// assign 50 % of the other indoor time to outdoors
							outdoor = this
									.modelOutdoorConcPop((int) (mtus.get_time(
											j, "outdoor") + mtus.get_time(j,
											"other")), 1, i_urb[j], mtus
											.get_groupID(j));
							travel = this
									.modelOutdoorConcPop(mtus.get_time(j,
											"travel"), e, i_urb[j], mtus
											.get_groupID(j));
							float exp_all = home + work + travel + outdoor;

							// get indoor and outdoor part of the exposure
							float exp = exp_all;
	
							// INSERT INTO exposure_country_realisation
							// (simulation_id, diary_id, group_id, iteration,
							// value)
							String[] tmp = { sim.get_simID(),
									mtus.get_diaryID(j), mtus.get_group(j),
									(i + 1 + ""), (exp + "") };
							individualResults.add(tmp);
						} catch (NullPointerException npe) {
							LogFile.logger.log(Level.SEVERE,
									"Exception during exposure calculation for individual: \n "
											+ mtus.get_diaryID(j) + ", "
											+ npe.toString());
						}
					}
					try {
						// write individual results to database
						df.insertRows(individualResults, ind_results_columns,
								"exposure_country_realisation");
					} catch (Exception ex) {
						LogFile.logger.log(Level.SEVERE,
								"Exception during inserting exposure country realisations into table \n "
										+ ex.toString() + " Cause "
										+ ex.getCause());
					}
				}
				if ((i + 1) % 10 == 0) // only log every tenth iteration
					console("Iteration: " + (i + 1));
			}
		} catch (Exception ex) {
			LogFile.logger
					.log(Level.SEVERE,
							"Exception during calculating exposure \n "
									+ ex.toString());
			// throw ex;
		}
	}

	public float[][] executeModelCsv() throws OutOfMemoryError {
		parameter_realisations = new ArrayList<String[]>();
		// get simulation parameters
		int iterations = sim.get_iterations();
		boolean gridbased = sim.get_gridbased();
		boolean individualResults = sim.get_individualResults();
		boolean subgroupResults = sim.get_subgroupResults();
		boolean totalPopResults = sim.get_totalPopResults();
		int cellCount = emep.get_concentrationArray().size();
		int sampleSize;
		String[] subgroups = sim.get_subgroup_array();
		float[] f_cig = home.get_parameter("f_cig");
		float f_bio = home.get_parameter("f_bio")[0];
		// result matrix
		float[][] results, i_urb;
		total_pop = this.sumPop();
		sum_pop = 0;
		for(int i=0; i<total_pop.length; i++){
			sum_pop += total_pop[i];
		}
		if (gridbased) {
			// gridbased results repeat each cellID 8 times for each subgroups
			if(subgroupResults)
				results = new float[cellCount * 8][iterations];
			else 
				results = new float[cellCount][iterations];
			gridbasedIDs = new ArrayList<String[]>();
//			sampleSize=mtus.get_size();
//			i_urb = fractionSampling();
			sampleSize = mtus.get_subgroups().size();
			i_urb = new float[sampleSize][emep.get_size()];
			for(int i=0; i<emep.get_size(); i++){					
				i_urb[0][i] = emep.get_urban_population_percentage(i) * emep.get_urban_increment(i);
			}
		} 
		else if(individualResults){
			sampleSize = mtus.get_size();
			results = new float[sampleSize][iterations];
			i_urb = fractionSampling();
		}
		else{
			sampleSize = mtus.get_subgroups().size();
			if(subgroupResults)
				results = new float[sampleSize][iterations];	
			else 
				results = new float[1][iterations];
			i_urb = new float[sampleSize][emep.get_size()];
			for(int i=0; i<emep.get_size(); i++){					
				i_urb[0][i] = emep.get_urban_population_percentage(i) * emep.get_urban_increment(i);
			}
		}

		// **** RUN SIMULATION ****
		// loop through realisations
		for (int i = 0; i < iterations; i++) {
			// sample parameter values for this realisation
			float a_work = sampling(work.get_parameter("a_work"), work
					.get_parameter_distribution("a_work"),false);
			float a_home = sampling(home.get_parameter("a_home"), home
					.get_parameter_distribution("a_home"),false);
			float v_work = sampling(work.get_parameter("v_work"), work
					.get_parameter_distribution("v_work"),false);
			float v_home = sampling(home.get_parameter("v_home"), home
					.get_parameter_distribution("v_home"),false);
			float p_work = sampling(work.get_parameter("p_work"), work
					.get_parameter_distribution("p_work"),false);
			float p_home = sampling(home.get_parameter("p_home"), home
					.get_parameter_distribution("p_home"),true);
			float k = sampling(home.get_parameter("k_home"), home
					.get_parameter_distribution("k_home"),false);
			float e = sampling(travel.get_parameter("e_travel"), travel
					.get_parameter_distribution("e_travel"),false);
			float s_bgr = sampling(home.get_parameter("s_bgr"), home
					.get_parameter_distribution("s_bgr"),false);
			float s_cig = sampling(home.get_parameter("s_cig"), home
					.get_parameter_distribution("s_cig"),false);
			float s_bio = sampling(home.get_parameter("s_bio"), home
					.get_parameter_distribution("s_bio"),false);
			// dividing n_cig by 24 to get number of cig smoked per hour
			float n_cig = sampling(home.get_parameter("n_cig"), home
					.get_parameter_distribution("n_cig"),false) / 24;

			// if calculation is without indoor sources, set sources to 0
			if(sim.get_exBackground())
				s_bgr = 0;
			if(sim.get_exBiomass())
				s_bio = 0;
			if(sim.get_exETS())
				s_cig = 0;

			// add realisation values to array
			parameter_realisations.add(buildParString("a", "home", i, a_home));
			parameter_realisations.add(buildParString("a", "work", i, a_work));
			parameter_realisations.add(buildParString("v", "home", i, v_home));
			parameter_realisations.add(buildParString("v", "work", i, v_work));
			parameter_realisations.add(buildParString("p", "home", i,
					p_home));
			parameter_realisations.add(buildParString("p", "work", i,
					p_work));
			parameter_realisations.add(buildParString("k", "all", i, k));
			parameter_realisations.add(buildParString("e", "travel", i, e));
			parameter_realisations
					.add(buildParString("s_bgr", "home", i, s_bgr));
			parameter_realisations
					.add(buildParString("s_bio", "home", i, s_bio));
			parameter_realisations
					.add(buildParString("s_cig", "home", i, s_cig));
			parameter_realisations
					.add(buildParString("n_cig", "home", i, n_cig));

			// assign only a certain percentage of the time spend at home for cigarettes consumption
			float s_cig_adj = s_cig * sim.get_ratioETS();
			
			// **** LOOP THROUGH GRID CELLS ****
			if (gridbased) {			
				int resCount = 0;
				for (int cellID = 0; cellID < cellCount; cellID++) {
					float totalResults = 0;
					float cellPop = 0;
					for(int s=0; s<mtus.get_subgroups().size(); s++){
						cellPop += emep.get_populationArray().get(cellID)[s];
					}
					// loop through subgroups
					for(int s=0; s<mtus.get_subgroups().size(); s++){
						
						float s_home = s_bgr + f_cig[s] * s_cig_adj * n_cig
						+ f_bio * s_bio;
					
						// calculate exposure per subgroup and grid cell
						float home = this.modelIndoorConcGridCell((int) (mtus.get_time_subgroup(
							s, "home") + 0.5 * mtus.get_time_subgroup(s, "other")),
							a_home, k, p_home, v_home, s_home, i_urb[0][cellID], cellID);
						float work = this.modelIndoorConcGridCell(mtus.get_time_subgroup(s,
							"work"), a_work, k, p_work, v_work, s_bgr, i_urb[0][cellID], cellID);
				
						// assign 50 % of the other indoor time to outdoors		
						float outdoor = this.modelOutdoorConcGridCell(
								(int) (mtus.get_time_subgroup(s, "outdoor") + 0.5* mtus.get_time_subgroup(
								s, "other")), 1, i_urb[0][cellID], cellID);				
						float travel = this.modelOutdoorConcGridCell(mtus.get_time_subgroup(s,
						"travel"), e, i_urb[0][cellID], cellID);				
						float exp_all = home + work + travel + outdoor;
						if(subgroupResults)
							results[resCount][i] = exp_all;		
						else{
							// sum subgroup results for total pop results
							totalResults += emep.get_populationArray().get(cellID)[s]*exp_all/cellPop;			
						}
							
						resCount++;
						String[] tmp = { emep.get_cellID(cellID), subgroups[s] };
						//only for one iteration
						if(i==0&&subgroupResults)
							gridbasedIDs.add(tmp);						
					}	
					// if total population results are required
					if(!subgroupResults){
						results[cellID][i] = totalResults;
						String[] tmp = { emep.get_cellID(cellID)};					
						if(i==0)
							gridbasedIDs.add(tmp);
					}
				}
			}

			// **** LOOP THROUGH INDIVIDUALS OR SUBGROUPS ****
			else {
				float totalResults = 0;
				for (int j = 0; j < sampleSize; j++) {		
					// **** if individual diaries should be used
					if(individualResults){
						// calculate home source for each individual depending if
						// biomass or smoking occurs
						float s_home = s_bgr + (float) smokers[j] * s_cig_adj * n_cig
							+ (float) biomass_burners[j] * s_bio;	
						// get exposure per environment
						// assign 50 % of the other indoor time to indoors
						float home = this.modelIndoorConcPop((int) (mtus.get_time(
								j, "home") + 0.5 * mtus.get_time(j, "other")),
								a_home, k, p_home, v_home, s_home, i_urb[j], mtus
										.get_groupID(j));
						float work = this.modelIndoorConcPop(mtus.get_time(j,
								"work"), a_work, k, p_work, v_work, s_bgr,
								i_urb[j], mtus.get_groupID(j));
						// assign 50 % of the other indoor time to outdoors
						float outdoor = this.modelOutdoorConcPop(
								(int) (mtus.get_time(j, "outdoor") + 0.5 * mtus.get_time(
										j, "other")), 1, i_urb[j], mtus
										.get_groupID(j));
						float travel = this.modelOutdoorConcPop(mtus.get_time(j,
								"travel"), e, i_urb[j], mtus.get_groupID(j));
						float exp_all = home + work + travel + outdoor;
						results[j][i] = exp_all;
					}
					// **** if subgroup averages should be used
					else{
						float s_home = s_bgr + f_cig[j] * s_cig_adj * n_cig
							+ f_bio * s_bio;
						// get exposure per environment
						// assign 50 % of the other indoor time to indoors
						float home = this.modelIndoorConcPop((int) (mtus.get_time_subgroup(
								j, "home") + 0.5 * mtus.get_time_subgroup(j, "other")),
								a_home, k, p_home, v_home, s_home, i_urb[0], j);
						float work = this.modelIndoorConcPop(mtus.get_time_subgroup(j,
								"work"), a_work, k, p_work, v_work, s_bgr,
								i_urb[0], j);
						// assign 50 % of the other indoor time to outdoors
						float outdoor = this.modelOutdoorConcPop(
								(int) (mtus.get_time_subgroup(j, "outdoor") + 0.5 * mtus.get_time_subgroup(
										j, "other")), 1, i_urb[0], j);
						float travel = this.modelOutdoorConcPop(mtus.get_time_subgroup(j,
								"travel"), e, i_urb[0], j);
						float exp_all = home + work + travel + outdoor;
						if(subgroupResults)
							results[j][i] = exp_all;	
						// sum subgroup results for total pop
						else
							totalResults += total_pop[j]*exp_all/sum_pop;							
					}
					// if total population results are required
					if(!subgroupResults)
						results[0][i] = totalResults;
				}
			}
			if ((i + 1) % 1000 == 0) // only log every tenth iteration
				console("Iteration: " + (i + 1));
		}
		/*
		 * } catch(Exception ex){LogFile.logger.log(Level.SEVERE,
		 * "Exception during calculating exposure \n "
		 * +ex.toString()+" Cause "+ex.getCause()); throw ex; }
		 */
		
		return results;
	}

	// **** SAMPLING METHODS ****
	// sampling of single values from a distribution (norm, lnorm or unif)
	private float sampling(float[] values, String distribution, boolean p) {
		float result = -1F;

		// ensure that no negative samples are drawn
		if (distribution.startsWith("lognorm")) {
			while(result<0||(""+result).equals("NaN")){
				result = LognormalDistribution
						.drawSampleFloat(values[0], values[1]);
				if(p&&result>1)	// for penetration factor only take values smaller than 1
					result = -1;
			}
		} else if (distribution.startsWith("norm")) {
			while(result<0||(""+result).equals("NaN")){
				result = NormalDistribution.drawSampleFloat(values[0], values[1]);
				if(p&&result>1)	// for penetration factor only take values smaller than 1
					result = -1;
			}
		} else if (distribution.startsWith("unif")) {
			if(values[0]==values[1])	// for sensitivity analysis min and max are the same
				result = values[0];
			while(result<0||(""+result).equals("NaN")){
				result = UniformDistribution.drawSampleFloat(values[0], values[1]);
			}
		}

		return result;
	}

	private float[] binomialArraySampling(ArrayList<Float> probabilities) {
		float[] result = new float[probabilities.size()];
		for (int i = 0; i < probabilities.size(); i++) {
			if (probabilities.get(i) > 0)
				result[i] = BinomialDistribution.drawSampleFloat(probabilities
						.get(i));
			else
				result[i] = 0;
		}
		return result;
	}

	// method for binomial sampling for fractions of smokers, biomass burners
	// and urban inhabitants
	private float[][] fractionSampling() throws OutOfMemoryError {
		float[][] urbinc_cells = null;
		try {
			// probability of biomass burning and smoking households
			// these assignments are constant over the simulations
			float[] f_cig = home.get_parameter("f_cig");
			float f_bio = home.get_parameter("f_bio")[0];
			float group_cig = 0F;
			biomass_burners = new float[mtus.get_size()];
			smokers = new float[mtus.get_size()];

			// urban increment population proportion for each cell
			ArrayList<Float> p_urb = emep
					.get_urban_population_percentageArray();
			try {
				urbinc_cells = new float[mtus.get_size()][emep
						.get_cellIDArray().size()];
			} catch (OutOfMemoryError ome) {
				LogFile.logger.log(Level.SEVERE,
						"Java out of memory exception during sampling from parameter fractions. \n "
								+ ome.toString());
				console("Exception in thread main java.lang.OutOfMemoryError: Java heap space");
				console("Number of iterations is too large for calculation. Please reduce iteration count.");
				throw ome;
			}
			// loop through all individuals to get probabilities for each
			// subgroup
			for (int i = 0; i < mtus.get_size(); i++) {
				// probability for individual in each grid cell to live in an
				// urban area
				float[] temp = binomialArraySampling(p_urb);
				urbinc_cells[i] = temp;

				// biomass
				biomass_burners[i] = BinomialDistribution
						.drawSampleFloat(f_bio);

				// smoker depending on subgroup
				if (mtus.get_groupID(i) > -1)
					group_cig = f_cig[mtus.get_groupID(i)];
				else
					group_cig = 0.5F;

				smokers[i] = BinomialDistribution.drawSampleFloat(group_cig);

			}
		} catch (Exception ex) {
			LogFile.logger.log(Level.SEVERE,
					"Exception during sampling from parameter fractions. \n "
							+ ex.toString() + " Cause " + ex.getCause());
			// throw ex;
		}
		return urbinc_cells;
	}

	// method for sampling from parameter distributions, returns one realisation
	// for each parameter
	private Hashtable<String, Float> distributionSampling() {
		Hashtable<String, Float> realisations = new Hashtable<String, Float>();
		try {
			// sample parameter values for this realisation
			float a_work = sampling(work.get_parameter("a"), work
					.get_parameter_distribution("a"),false);
			float a_home = sampling(home.get_parameter("a"), home
					.get_parameter_distribution("a"),false);
			float v_work = sampling(work.get_parameter("v"), work
					.get_parameter_distribution("v"),false);
			float v_home = sampling(home.get_parameter("v"), home
					.get_parameter_distribution("v"),false);
			float p_work = sampling(work.get_parameter("p_work"), work
					.get_parameter_distribution("p_work"),false);
			float p_home = sampling(home.get_parameter("p_home"), home
					.get_parameter_distribution("p_home"),true);
			float k = sampling(home.get_parameter("k"), home
					.get_parameter_distribution("k"),false);
			float e = sampling(home.get_parameter("e"), home
					.get_parameter_distribution("e"),false);
			float s_bgr = sampling(home.get_parameter("s_bgr"), home
					.get_parameter_distribution("s_bgr"),false);
			float s_cig = sampling(home.get_parameter("s_cig"), home
					.get_parameter_distribution("s_cig"),false);
			float s_bio = sampling(home.get_parameter("s_bio"), home
					.get_parameter_distribution("s_bio"),false);
			// dividing n_cig by 24 to get number of cig smoked per hour
			float n_cig = sampling(home.get_parameter("n_cig"), home
					.get_parameter_distribution("n_cig"),false) / 24;

			// put realisations to Hashtable
			realisations.put("a_work", a_work);
			realisations.put("a_home", a_home);
			realisations.put("v_work", v_work);
			realisations.put("v_home", v_home);
			realisations.put("p_work", p_work);
			realisations.put("p_home", p_home);
			realisations.put("k", k);
			realisations.put("e", e);
			realisations.put("s_bgr", s_bgr);
			realisations.put("s_bio", s_bio);
			realisations.put("s_cig", s_cig);
			realisations.put("n_cig", n_cig*24);
		} catch (Exception ex) {
			LogFile.logger.log(Level.SEVERE,
					"Exception during sampling from parameter distributions. \n "
							+ ex.toString() + " Cause " + ex.getCause());
			// throw ex;
		}
		return realisations;
	}

	// **** CONCENTRATION MODEL METHODS ****
	// models for individual based execution
	// indoor model
	private Float modelIndoorConcPop(int t1, float a, float k, float p,
			float v, float s, float[] p_urb, int groupID) {
		// array with total concentration and indoor and outdoor part
		Float res = 0F;
		try {
			float temp = 0;
			// System.out.println("Indoor Source Strength: "+s);
			// time conversion into fraction of the day
			float t = ((float) t1) / 1440.0F;

			// do the indoor model for all grid cells
			if (t != 0) {
				for (int i = 0; i < emep.get_concentrationArray().size(); i++) {
					// total concentration
					float cp = t
							* (emep.get_concentration(i) + (float) p_urb[i]
									* emep.get_urban_increment(i)) * p * a
							/ (a + k) + t * s / ((a + k) * v);

					// add to sum over grid cells
					//if (groupID < 8 && groupID >= 0) // ignore individual if no
														// subgroup is available
						temp += (cp * emep.get_population(i, groupID));
				}
			}
			// divide by total population to get average
			res = temp / total_pop[groupID];
		} catch (Exception ex) {
			LogFile.logger.log(Level.SEVERE,
					"Exception during calculating indoor individual exposure \n "
							+ ex.toString());
			// throw ex;
		}
		return res;
	}

	// outdoor model
	private Float modelOutdoorConcPop(int t1, float e, float[] p_urb, int groupID) {
		Float concPop = 0F;
		try {
			float temp = 0;
			// get fraction of the day
			float t = ((float) t1) / 1440.0F;

			// if no time is spent in this microenvironment, return 0
			if (t == 0)
				return 0F;

			// do the outdoor model for all grid cells
			else {
				for (int i = 0; i < emep.get_concentrationArray().size(); i++) {
					// calculate outdoor concentration with urban increment and
					// traffic enrichment factor
					float cp = t
							* e
							* (emep.get_concentrationArray().get(i) + (float) p_urb[i]
									* emep.get_urban_increment(i));

					// add to sum over grid cells
					temp = +cp * emep.get_population(i, groupID);
				}
			}
			// divide by total population to get average
			concPop = temp / total_pop[groupID];
		} catch (Exception ex) {
			LogFile.logger.log(Level.SEVERE,
					"Exception during calculating outdoor individual exposure \n "
							+ ex.toString());
		}
		return concPop;
	}

	// models for grid based execution
	// indoor model
	private float modelIndoorConcGridCell(int t1, float a, float k, float p,
			float v, float s, float p_urb, int cellID) {
		// array with exposure per subgroup for this grid cell
		float res = 0F;
		try {
			// time conversion into fraction of the day
			float t = ((float) t1) / 1440.0F;

			// total concentration
			res = t * (emep.get_concentration(cellID) + p_urb
							* emep.get_urban_increment(cellID)) * p * a
					/ (a + k) + t * s / ((a + k) * v);
				
		// multiply with population
		// res[i] = (temp[i]/count[i])*emep.get_population(cellID, i);
			
		} catch (Exception ex) {
			LogFile.logger.log(Level.SEVERE,
					"Exception during calculating indoor grid exposure \n "
							+ ex.toString());
		}
		return res;
	}
	
	// outdoor model
	private float modelOutdoorConcGridCell(int t1, float e, float p_urb, int cellID) {
		// array with exposure per subgroup for this grid cell
		float res = 0F;
		try {
			// time conversion into fraction of the day
			float t = ((float) t1) / 1440.0F;

			// total concentration
			res = t * e * (emep.get_concentrationArray().get(cellID) + p_urb
					* emep.get_urban_increment(cellID));
				
		// multiply with population
		// res[i] = (temp[i]/count[i])*emep.get_population(cellID, i);
			
		} catch (Exception ex) {
			LogFile.logger.log(Level.SEVERE,
					"Exception during calculating outdoor grid exposure \n "
							+ ex.toString());
		}
		return res;
	}
	
	// indoor model
	private float[] modelIndoorConcGrid(String me, float a, float k, float p,
			float v, float s, float[][] p_urb, int cellID) {
		// array with exposure per subgroup for this grid cell
		float[] res = new float[8];
		try {
			float[] temp = new float[8];
			int[] count = new int[8];

			// do the indoor model for all individuals
			for (int i = 0; i < mtus.get_size(); i++) {

				// get microenvironment time of individual i
				float t1;
				// for home environment add 50 % of other
				if (me.equals("home"))
					t1 = (float) (mtus.get_time(i, me) + 0.5 * mtus.get_time(i,
							"other"));
				else
					t1 = (float) mtus.get_time(i, me);

				// time conversion into fraction of the day
				float t = ((float) t1) / 1440.0F;

				// total concentration
				float c = t
						* (emep.get_concentration(cellID) + (float) p_urb[i][cellID]
								* emep.get_urban_increment(cellID)) * p * a
						/ (a + k) + t * s / ((a + k) * v);

				// add exposure to sum of the respective subgroup
				temp[mtus.get_groupID(i)] += c;
				count[mtus.get_groupID(i)] += 1;
			}

			// divide by number of individuals per subgroup to get average
			for (int i = 0; i < 8; i++) {
				if (count.length == 8 && count[i] != 0)
					res[i] = temp[i] / count[i];
				else {
					res[i] = 0;
					// System.out.println("Grid cell: "+cellID+", group: "+(i+1)+", c: "+temp[i]+", pop: "+count[i]);
				}
				// multiply with population
				// res[i] = (temp[i]/count[i])*emep.get_population(cellID, i);
			}
		} catch (Exception ex) {
			LogFile.logger.log(Level.SEVERE,
					"Exception during calculating indoor grid exposure \n "
							+ ex.toString());
		}
		return res;
	}

	// outdoor model
	private float[] modelOutdoorConcGrid(String me, float e, float[][] p_urb,
			int cellID) {
		// array with exposure per subgroup for this grid cell
		float[] res = new float[8];
		try {
			float[] temp = new float[8];
			int[] count = new int[8];

			// do the outdoor model for all individuals
			for (int i = 0; i < mtus.get_size(); i++) {

				// get microenvironment time of individual i
				float t1;
				// for outdoor environment add 50 % of other
				if (me.equals("outdoor"))
					t1 = (float) (mtus.get_time(i, me) + 0.5 * mtus.get_time(i,
							"other"));
				else
					t1 = (float) mtus.get_time(i, me);

				// time conversion into fraction of the day
				float t = ((float) t1) / 1440.0F;

				// total concentration
				float c = t
						* e
						* (emep.get_concentration(cellID) + (float) p_urb[i][cellID]
								* emep.get_urban_increment(cellID));

				// add exposure to sum of the respective subgroup
				temp[mtus.get_groupID(i)] += c;
				count[mtus.get_groupID(i)] += 1;
			}
			int popSum = 0;
			// divide by number of individuals per subgroup to get average
			for (int i = 0; i < count.length; i++) {
				if (count.length == 8 && count[i] != 0)
					res[i] = temp[i] / count[i];
				else {
					res[i] = 0;
					// System.out.println("Grid cell: "+cellID+", group: "+(i+1)+", c: "+temp[i]+", pop: "+count[i]);
				}
				popSum += count[i];
				// multiply with population
				// res[i] = (temp[i]/count[i])*emep.get_population(cellID, i);
			}
		} catch (Exception ex) {
			LogFile.logger.log(Level.SEVERE,
					"Exception during calculating outdoor grid exposure \n "
							+ ex.toString());
		}
		return res;
	}

	// **** UTILITIES ****
	private float[] sumPop() {
		float[] sums = new float[8];
		try {
			for (int i = 0; i < emep.get_populationArray().size(); i++) {
				for (int j = 0; j < 8; j++) {
					sums[j] += emep.get_population(i, j);
				}
			}
		} catch (Exception e) {
			LogFile.logger.log(Level.SEVERE,
					"Exception during calculating population sum \n "
							+ e.toString());
		}
		return sums;
	}

	private float[] sumPopGroups() {
		float[] sums = new float[8];
		try {
			for (int i = 0; i < emep.get_populationArray().size(); i++) {
				for (int j = 0; j < 8; j++) {
					sums[j] += emep.get_population(i, j);
				}
			}
		} catch (Exception e) {
			LogFile.logger.log(Level.SEVERE,
					"Exception during calculating population sum \n "
							+ e.toString());
		}
		return sums;
	}

	
	private String[] buildParString(String par, String me, int i, float val) {
		String[] str = { par, me, (i + 1 + ""), (val + "") };
		return str;
	}

	public ArrayList<String[]> get_parameterRealisations() {
		return parameter_realisations;
	}

	public float[] get_smokers() {
		return smokers;
	}

	public float[] get_biomass_burners() {
		return biomass_burners;
	}

	public ArrayList<String[]> get_gridbased_ids() {
		return gridbasedIDs;
	}

	private void console(String data) {
		// console output
		System.out.println(data);
	}

}
