package de.ifgi.exposure.statistics;

import java.util.Random;
import java.lang.Math;

public class NormalDistribution extends ProbabilityDistribution{
/**
 * Draws samples x.snd from standard normal distribution, m.snd=0, s.snd=1
 * Calculates probabilities for samples
 * f(x.snd)= 1/(s.snd*sqrt(2*pi))*exp(-0.5*((x.snd-m)/s.snd)^2)
 * 
 * Estimate x.new with f(x.snd) and new mu and sigma
 * f(x.snd) = 1/(sqrt(2*pi))*exp(-0.5*z^2)
 * f(x.snd)*(sqrt(2*pi)) = exp(-0.5*z^2)
 * log(f(x.snd)*(sqrt(2*pi))) = -0.5*z^2
 * -2*(log(f(x.snd)*(sqrt(2*pi)))) = z^2
 * sqrt(-2*(log(f(x.snd)*(sqrt(2*pi))))) = z
 * with z= (x.new-mu)/sigma
 * x.new = sigma*(sqrt(-2*(log(fx*(sqrt(2*pi))))))+mu
 */
	private double mu, sigma;
	
	public NormalDistribution(){
		// constructor only for child class lognormal distribution
	}
	
	public NormalDistribution(double mean, double sd){
		mu=mean;
		sigma=sd;
	}
	
	public double drawSample(){
		double result; double standardnormal;
		Random r = new Random();
		
		// get sample from standard normal distribution
		standardnormal = r.nextGaussian();
		
		// convert this sample to a sample from the desired distribution
		double fx = sndProbability(standardnormal);
		
		// normal distribution to standard normal distribution
		result = sndProbabilityToSample(mu, sigma, fx);
		
		// results will all be above the mean as the distribution is symmetrical
		// move some of the values below the mean
		if(standardnormal<0)
			result = 2*mu-result;

		return result;
	}
	
	public double[] drawSamples(int n){
		double[] result = new double[n]; double[] standardnormal = new double[n];
		Random r = new Random();
		
		for(int i=0; i<n; i++){
		// get sample from standard normal distribution
		standardnormal[i] = r.nextGaussian();
		
		// convert this sample to a sample from the desired distribution
		double fx = sndProbability(standardnormal[i]);
		
		// normal distribution to standard normal distribution
		result[i] = sndProbabilityToSample(mu, sigma, fx);
		
		// results will all be above the mean as the distribution is symmetrical
		// move some of the values below the mean
		if(standardnormal[i]<0)
			result[i] = 2*mu-result[i];
		
		}	
		return result;
	}
	
	// static class methods for drawing samples
	public static double drawSampleDouble(double mean, double sd){
		double result; double standardnormal;
		Random r = new Random();
		
		// get sample from standard normal distribution
		standardnormal = r.nextGaussian();
		
		// convert this sample to a sample from the desired distribution
		double fx = sndProbability(standardnormal);
		
		// normal distribution to standard normal distribution
		result = sndProbabilityToSample(mean, sd, fx);
		
		// results will all be above the mean as the distribution is symmetrical
		// move some of the values below the mean
		if(standardnormal<0)
			result = 2*mean-result;
	
		return result;
	}
	
	public static float drawSampleFloat(float mean, float sd){
		float result; float standardnormal;
		Random r = new Random();
		
		// get sample from standard normal distribution
		standardnormal = (float) r.nextGaussian();
		
		// convert this sample to a sample from the desired distribution
		float fx = (float)sndProbability(standardnormal);
		
		// normal distribution to standard normal distribution
		result = (float)sndProbabilityToSample(mean, sd, fx);
		
		// results will all be above the mean as the distribution is symmetrical
		// move some of the values below the mean
		if(standardnormal<0)
			result = 2*mean-result;
	
		return result;
	}
	
	// functions
	public static double sndProbability(double x){
		double fx = 1/(Math.sqrt(2*Math.PI))*Math.exp(-0.5*Math.pow(x,2));
		return fx;
	}
	
	public static double probability(double mean, double sd, double x){
		double fx = 1/(sd*Math.sqrt(2*Math.PI))*Math.exp(-0.5*Math.pow(((x-mean)/sd),2));
		return fx;
	}
	
	public static double sndProbabilityToSample(double mean, double sd, double fx){
		double result = sd*(Math.sqrt(-2*(Math.log(fx*(Math.sqrt(2*Math.PI))))))+mean;
		return result;	
	}
	
}
